﻿using System;
using System.Diagnostics;
using System.IO;

namespace ScriptCaching
{
    internal class ScriptNode
    {
        public void Active(bool active) => active_count_ += active ? 1 : -1;
        public bool IsLoaded => handler_ != null;
        public bool IsActive => active_count_ > 0;
        public string ProcName { get; private set; }
        public string ScriptName { get; private set; }

        // Module containing executable script code
        private WinWrap.Basic.Module module_;
        // Handler validates the function prototype during parsing and provides evaluation at run-time
        private WinWrap.Basic.Handler handler_;
        // FastEvaluate0 provides fast evaluation at run-time
        private WinWrap.Basic.FastEvaluate0 fast_evaluate_;
        // active count
        private int active_count_;

        public ScriptNode(string script_name)
        {
            ScriptName = script_name;
            // proc name is the script name's final part without the extension
            ProcName = Path.GetFileNameWithoutExtension(script_name);
        }

        public object Evaluate(object default_value = null)
        {
            // the handler exists if the script defines Function <ProcName>() As Variant
            if (!handler_.Exists)
                return default_value;

            if (fast_evaluate_ != null)
                return fast_evaluate_.Evaluate();

            return handler_.Evaluate();
        }

        public void Load(WinWrap.Basic.IBasicNoUI basicNoUI, bool fast)
        {
            // create a handler for accessing the sub
            handler_ = basicNoUI.CreateHandler($"{ScriptName}|Function {ProcName}() As Variant '#Handles Optional");
            Debug.Assert(handler_ != null);
            // load the module -- public symbols are only accessible via module_
            module_ = basicNoUI.ModuleInstance(ScriptName, false);
            if (module_ == null)
            {
                // syntax error
                handler_.Dispose();
                handler_ = null;
                // turn the error into an exception and throw it
                throw basicNoUI.Error.Exception;
            }

            if (fast)
                fast_evaluate_ = handler_.Fast as WinWrap.Basic.FastEvaluate0;
        }

        public void Unload()
        {
            fast_evaluate_ = null;
            // get rid of the module
            module_?.Dispose();
            module_ = null;
            // get rid of the handler
            handler_?.Dispose();
            handler_ = null;
        }
    }
}
